@@ -25,12 +25,22 @@ import jsonschema
 from mypy.checker import Generic
 from ruamel.yaml import YAML
 
-from .steps import Target
 from .validation import validate
 
 T = TypeVar('T')
 
 
+@dataclass(frozen=True)
+class Target(Enum):
+    def __eq__(self, other):
+        return self.value == other.value
+
+    PULL_REQUEST = 'PullRequest'
+    PULL_REQUEST_BASE = 'PullRequestBase'
+    ACCEPTANCE = 'Acceptance'
+    PRODUCTION = 'Production'
+
+
 @dataclass(frozen=True)
 class Stage(Enum):
     def __eq__(self, other):
@@ -316,7 +326,9 @@ def load_project(root_dir, project_path: str, strict: bool = True) -> Project:
                 schema = yaml.load(template.decode('utf-8'))
                 validate(yaml_values, schema)
 
-            return Project.from_yaml(yaml_values, project_path)
+            project = Project.from_yaml(yaml_values, project_path)
+            logging.debug(f'Loaded project {project.path}')
+            return project
         except jsonschema.exceptions.ValidationError as exc:
             logging.warning(f'{project_path} does not comply with schema: {exc.message}')
             raise
